
/* U_PL_HP.C : Plot driver for HPGL, used with U_PLOT                */
/* Supported Devices: HP-Laserjet 4   (PCL5 with HPGL)               */
/*                    HP LAserjet 2P with PacificPage HPGL-Cartridge */
/*                    HP 7550 Plotter                                */
/* Supported Formats:                                                */
/*      DIN A4 Portrait and Landscape, autofeed     (all Devices)    */
/*      DIN A4 Portrait and Landscape, NO autofeed  (Plotter only)   */
/*      DIN A3 Portrait and Landscape, NO autofeed  (Plotter only)   */
/* Portrait and Landscape is handled internally, without "RO 90;"    */ 

/**************** Changes ******************************************/
/* AV  31.04.94: Direct output to Queue on VMS-Systems if filename */
/*               starts with |   (like a pipe on Unix)             */
/*******************************************************************/
#include "u_pl_loc.h"
#include "u_plot.h"
#include "v_inp.h"

#include <stdio.h>
#include <string.h>

#define DEFCOLOR 1
#define MAXCOLOR 9

typedef enum formats {DINA4,DINA3,FREE_FORMAT} formats;
typedef enum orients {PORTRAIT,LANDSCAPE} orients;

/* We use max. 27 cm on DIN A4, 38 cm on DIN A3 */
#define DINA4X 10800
#define DINA4Y  7600
#define DINA3X 15200
#define DINA3Y 10800

#define ESC 27
#define FF  12

static FILE *plotfile;
static int file_is_open = FALSE; 
static int plotted, is_graphmode;
static int devsizex, devsizey, pen, devicetype;
static formats format;
static orients orient;
static int rotate = FALSE;

static int output_is_pipe = FALSE;

#ifdef VMS
static char outputqueue[80];
#define TEMPFILENAME "HARDCOPY.HPL"
#endif

static void write_point(char *cmd, int x, int y){
   int tmp;
   if (rotate) { tmp = y; y = devsizex - x; x = tmp; }
   fprintf(plotfile,"%s %1d,%1d;",cmd,x,y);
}

static int plotd_setfg_hpgl(int handle, int fg){
   if (fg==PLOT_USEDEFAULT) fg=DEFCOLOR;
   fg = fg % MAXCOLOR;
   if (fg==0) fg=DEFCOLOR;
   if (fg == pen) return(pen);
   pen = fg;
   fprintf(plotfile,"SP %1d;\n",pen);
   return(pen);
}

static int plotd_getfg_hpgl(int handle){
   return(pen); }

static int plotd_setbg_hpgl(int handle, int bg){
   return(0);
}

static int plotd_getbg_hpgl(int handle){
   return(0);
}

static void plotd_settextalign_hpgl(int handle, int xalign, int yalign){
   int origin;
   if (xalign==PLOT_LEFTTEXT)   origin = 1;
   if (xalign==PLOT_CENTERTEXT) origin = 4;
   if (xalign==PLOT_RIGHTTEXT)  origin = 7;
   if (yalign==PLOT_CENTERTEXT) origin = origin + 1;
   if (yalign==PLOT_TOPTEXT)    origin = origin + 2;
   fprintf(plotfile,"LO %1d\n",origin);
}


static void plotd_graphmode_hpgl(int handle){
   if (is_graphmode) return;
   is_graphmode = TRUE;
   plotted = FALSE;
   pen = 1;
   fprintf(plotfile,"IN; SP 1;");
   fprintf(plotfile,"PA;PU 0,0; LO 1;\n");
   plotd_setfg_hpgl(handle,DEFCOLOR);
   if (rotate) fprintf(plotfile,"DI 0,-1;\n");
}

static int plotd_modify_hpgl(int nh, handletyp *h, int sizex, int sizey,
  		      int devtype, int paper, char name[]){
   int autofeed;
   /* if not specified, check desired format */
   format = DINA4; orient=LANDSCAPE; autofeed=FALSE;
   devicetype = devtype;
   switch (paper){
       case PLOT_PAPER_A4P_AUTO : orient = PORTRAIT;
       case PLOT_PAPER_A4L_AUTO : autofeed = TRUE; break;
       case PLOT_PAPER_A4P : orient = PORTRAIT;
       case PLOT_PAPER_A4L : break;
       case PLOT_PAPER_A3P : orient = PORTRAIT;
       case PLOT_PAPER_A3L : format = DINA3; break;
       case PLOT_PAPER_FREE : format = FREE_FORMAT; break;
       default:
         if (sizex > sizey){ orient = LANDSCAPE;
           if ((sizex > DINA4X) || (sizey > DINA4Y)) format = DINA3; 
         } else { orient = PORTRAIT;
           if ((sizex > DINA4Y) || (sizey > DINA3X)) format = DINA3;
         } 
         if (format==DINA4) autofeed = TRUE;
   } /* switch...*/

   /* Only Din-A4 autofeed on Laserprinters */ 
   if ((devtype==PLOT_DEV_HPGL_LJ4) || (devtype==PLOT_DEV_HPGL_LJ2PP)){
      autofeed=TRUE; format=DINA4;}
   rotate = FALSE;
   if ((orient==PORTRAIT) && (devtype==PLOT_DEV_HPGL_7550)) rotate = TRUE;
   if ((orient==PORTRAIT) && (devtype==PLOT_DEV_HPGL_LJ2PP)) rotate = TRUE;
   if ((orient==LANDSCAPE) && (devtype==PLOT_DEV_HPGL_LJ4)) rotate = TRUE;
   
   if (orient==LANDSCAPE){
     if (format==DINA4){ sizex = DINA4X; sizey = DINA4Y; }
     if (format==DINA3){ sizex = DINA3X; sizey = DINA3Y; }
   } else {
     if (format==DINA4){ sizey = DINA4X; sizex = DINA4Y; }
     if (format==DINA3){ sizey = DINA3X; sizex = DINA3Y; }
   }
     
   if (devtype==PLOT_DEV_HPGL_LJ4){
      fprintf(plotfile,"%c%c0B\n",ESC,'%');
   }
   /* Initialize plotter, to switch off Rotation, Origin,... */
   if (devtype==PLOT_DEV_HPGL_7550) fprintf(plotfile,"IN;");
   if (! autofeed) {
     if (format==DINA3) 
       fprintf(plotfile,"WD%16s%16s\003NR;\n",proc_username(),"Load A3 paper");
     if (format==DINA4) 
       fprintf(plotfile,"WD%16s%16s\003NR;\n",proc_username(),"Load A4 paper");
   }
   devsizex = sizex; devsizey = sizey; pen = DEFCOLOR;
   plotd_setfg_hpgl(nh,DEFCOLOR);
   return 1;
}

static int plotd_getdevice_hpgl(int handle, int *sizex, int *sizey){
   *sizex = devsizex; *sizey = devsizey; return(PLOT_DEV_HPGL);
}

static void plotd_point_hpgl(int handle, int X, int Y){
   plotted = TRUE;
   write_point("PU",X,Y);
   fprintf(plotfile,"PD;\n");
}

static void plotd_line_hpgl(int handle, int X0, int Y0, int X1, int Y1){
   plotted = TRUE;
   write_point("PU",X0,Y0);
   write_point("PD",X1,Y1);
   fprintf(plotfile,"\n");
}

static void plotd_polyline_hpgl(int handle, int pts, int X[], int Y[]){
   int i;
   plotted = TRUE;
   write_point("PU",X[0],Y[0]);
   for (i=1; i<pts; i++){
      write_point("PD",X[i],Y[i]);
      if ((i % 5)==0) fprintf(plotfile,"\n");
   }
   if (((pts-1) % 5)!=0) fprintf(plotfile,"\n");
}

static void plotd_writetext_hpgl(int handle, int x, int y, char txt[]){
   plotted = TRUE;
   write_point("PU",x,y);
   fprintf(plotfile,"\nLB %s\003;\n",txt);
}

static void plotd_erasetext_hpgl(int handle, int x, int y, char txt[]){
/* just do nothing... */
}

static void plotd_clear_hpgl(int handle){
   if (! plotted) return;
   plotted = FALSE;
   if (devicetype==PLOT_DEV_HPGL_LJ4){
      fprintf(plotfile,"%c%c0B\n%c\n%c%c0B\nPU 0,0;\n",ESC,'%',FF,ESC,'%');
      return;
   }
   fprintf(plotfile,"PG; PU 0,0;\n");
}

static void plotd_textmode_hpgl(int handle){
/* just do nothing... */
   is_graphmode = FALSE;
}

static void plotd_close_hpgl(int handle){
/* To finish, throw out paper, put pen back to caroussel, initialize plotter */
/* and close the file or pipe                                                */
   int stat;
   if (plotted) fprintf(plotfile,"SP;PG;IN;\n");
   if (output_is_pipe){
#ifdef UNIX
      stat = pclose(plotfile);
#elif defined(VMS)
      fclose(plotfile);
      VMS_submit(outputqueue,TEMPFILENAME,TRUE);
#endif
   } else
   fclose(plotfile);
   file_is_open = FALSE;
}

static int plotd_setmode_hpgl(int handle, int mode){
   return(PLOT_MODE_OVER);
}

static int plotd_getmode_hpgl(int handle){
   return(PLOT_MODE_OVER);
}

static int plotd_getpointer_hpgl(int handle, int *x, int *y){
   return(PLOT_TRM_ABORT);
}

int plotd_init_hpgl(int nh, handletyp *h, int sizex, int sizey,
		     int posx, int posy, char name[]){
   char *p;
   if (file_is_open) return(PLOT_STAT_NOMOREDEV);
   plotted = FALSE; is_graphmode = FALSE;
   output_is_pipe = (name[0]=='|');
   if (output_is_pipe){
#ifdef UNIX
      plotfile = popen(name+1,"w");
#elif defined(VMS)
      plotfile = fopen(TEMPFILENAME,"w");
      strcpy(outputqueue,name+1);
      if ((p=strchr(outputqueue,'.')) != NULL) (*p) = '\0';
#endif
   } else plotfile = fopen(name,"w");
   if (! plotfile) return(PLOT_STAT_OPENERROR);
   file_is_open = TRUE;
   h->ploth_modify     =   plotd_modify_hpgl;
   h->ploth_getdevice  =   plotd_getdevice_hpgl;
   h->ploth_line       =   plotd_line_hpgl;
   h->ploth_point      =   plotd_point_hpgl;
   h->ploth_polyline   =   plotd_polyline_hpgl;
   h->ploth_setfg      =   plotd_setfg_hpgl;
   h->ploth_getfg      =   plotd_getfg_hpgl;
   h->ploth_setbg      =   plotd_setbg_hpgl;
   h->ploth_getbg      =   plotd_getbg_hpgl;
   h->ploth_writetext  =   plotd_writetext_hpgl;
   h->ploth_erasetext  =   plotd_erasetext_hpgl;
   h->ploth_graphmode  =   plotd_graphmode_hpgl;
   h->ploth_clear      =   plotd_clear_hpgl;
   h->ploth_textmode   =   plotd_textmode_hpgl;
   h->ploth_close      =   plotd_close_hpgl;
   h->ploth_setmode    =   plotd_setmode_hpgl;
   h->ploth_getmode    =   plotd_getmode_hpgl;
   h->ploth_checkevent =   plotd_getpointer_hpgl;
   h->ploth_getpointer =   plotd_getpointer_hpgl;
   h->ploth_settextalign = plotd_settextalign_hpgl;
   h->capabilities     = PLOT_CAPABILITY_HARDCOPY 
                       + PLOT_CAPABILITY_CM;
   h->cm_per_unit      = 0.0025;
   plotd_modify_hpgl(nh,h,sizex,sizey,posx,posy,name);
   return(PLOT_STAT_OK);
}
